@@ -25,7 +25,7 @@
 //!
 //! The actual combinators can be found in `parser.rs`. Various tricks are used to provide a nice
 //! API and work around limitations in the type system, such as the concrete types like
-//! `MapWithContext`, and the `make_parser_concrete` hack macro.
+//! `MapWithContext`.
 //!
 //! The actual parsers are then grouped into categories based loosely on the AML grammar sections in
 //! the ACPI spec. Most are written in terms of combinators, but some have to be written in a more
@@ -43,6 +43,7 @@ extern crate std;
 #[cfg(test)]
 mod test_utils;
 
+pub(crate) mod expression;
 pub(crate) mod misc;
 pub(crate) mod name_object;
 pub(crate) mod namespace;
@@ -51,9 +52,8 @@ pub(crate) mod parser;
 pub mod pci_routing;
 pub(crate) mod pkg_length;
 pub mod resource;
+pub(crate) mod statement;
 pub(crate) mod term_object;
-pub(crate) mod type1;
-pub(crate) mod type2;
 pub mod value;
 
 pub use crate::{namespace::*, value::AmlValue};
@@ -335,8 +335,9 @@ impl AmlContext {
                         field.read_field(self)
                     }
                     AmlType::BufferField => {
-                        log::info!("Trying to write to buffer field!");
-                        todo!()
+                        let mut buffer_field = self.namespace.get(handle).unwrap().clone();
+                        buffer_field.write_buffer_field(value.clone(), self)?;
+                        Ok(value)
                     }
                     typ => {
                         *self.namespace.get_mut(handle)? = value.as_type(typ, self)?;
@@ -789,6 +790,8 @@ pub enum AmlError {
     TypeCannotBeCompared(AmlType),
     /// Produced when the `Mid` operator is applied to a value of a type other than `Buffer` or `String`.
     TypeCannotBeSliced(AmlType),
+    TypeCannotBeWrittenToBufferField(AmlType),
+    BufferFieldIndexesOutOfBounds,
 }
 
 #[cfg(test)]
