@@ -22,7 +22,8 @@ pub enum Resource {
 pub fn resource_descriptor_list(descriptor: &AmlValue) -> Result<Vec<Resource>, AmlError> {
     if let AmlValue::Buffer(bytes) = descriptor {
         let mut descriptors = Vec::new();
-        let mut bytes = bytes.as_slice();
+        let buffer_data = bytes.lock();
+        let mut bytes = buffer_data.as_slice();
 
         while bytes.len() > 0 {
             let (descriptor, remaining_bytes) = resource_descriptor(bytes)?;
@@ -600,7 +601,7 @@ mod tests {
         ]
         .to_vec();
 
-        let value: AmlValue = AmlValue::Buffer(Arc::new(bytes));
+        let value: AmlValue = AmlValue::Buffer(Arc::new(spinning_top::Spinlock::new(bytes)));
         let resources = resource_descriptor_list(&value).unwrap();
 
         assert_eq!(
@@ -710,7 +711,7 @@ mod tests {
         ]
         .to_vec();
 
-        let value: AmlValue = AmlValue::Buffer(Arc::new(bytes));
+        let value: AmlValue = AmlValue::Buffer(Arc::new(spinning_top::Spinlock::new(bytes)));
         let resources = resource_descriptor_list(&value).unwrap();
 
         assert_eq!(
@@ -811,7 +812,7 @@ mod tests {
         ]
         .to_vec();
 
-        let value: AmlValue = AmlValue::Buffer(Arc::new(bytes));
+        let value: AmlValue = AmlValue::Buffer(Arc::new(spinning_top::Spinlock::new(bytes)));
         let resources = resource_descriptor_list(&value).unwrap();
 
         assert_eq!(
