@@ -6,6 +6,7 @@ use alloc::{
 };
 use bit_field::BitField;
 use core::{cmp, fmt, fmt::Debug};
+use spinning_top::Spinlock;
 
 #[derive(Clone, Copy, PartialEq, Eq, Debug)]
 pub enum RegionSpace {
@@ -197,9 +198,9 @@ pub enum AmlValue {
         flags: MethodFlags,
         code: MethodCode,
     },
-    Buffer(Arc<Vec<u8>>),
+    Buffer(Arc<Spinlock<Vec<u8>>>),
     BufferField {
-        buffer_data: Arc<Vec<u8>>,
+        buffer_data: Arc<Spinlock<Vec<u8>>>,
         /// In bits.
         offset: u64,
         /// In bits.
@@ -274,7 +275,6 @@ impl AmlValue {
         match self {
             AmlValue::Integer(value) => Ok(*value),
             AmlValue::Boolean(value) => Ok(if *value { u64::max_value() } else { 0 }),
-
             AmlValue::Buffer(ref bytes) => {
                 /*
                  * "The first 8 bytes of the buffer are converted to an integer, taking the first
@@ -283,7 +283,8 @@ impl AmlValue {
                  *
                  * XXX: Buffers with length `0` appear in real tables, so we return `0` for them.
                  */
-                let bytes = if bytes.len() > 8 { &bytes[0..8] } else { bytes };
+                let bytes = bytes.lock();
+                let bytes = if bytes.len() > 8 { &bytes[0..8] } else { &bytes[..] };
 
                 Ok(bytes.iter().rev().fold(0: u64, |mut i, &popped| {
                     i <<= 8;
@@ -291,22 +292,23 @@ impl AmlValue {
                     i
                 }))
             }
-
             /*
-             * Read from a field. This can return either a `Buffer` or an `Integer`, so we make sure to call
+             * Read from a field or buffer field. These can return either a `Buffer` or an `Integer`, so we make sure to call
              * `as_integer` on the result.
              */
             AmlValue::Field { .. } => self.read_field(context)?.as_integer(context),
+            AmlValue::BufferField { .. } => self.read_buffer_field(context)?.as_integer(context),
 
             _ => Err(AmlError::IncompatibleValueConversion { current: self.type_of(), target: AmlType::Integer }),
         }
     }
 
-    pub fn as_buffer(&self, context: &AmlContext) -> Result<Arc<Vec<u8>>, AmlError> {
+    pub fn as_buffer(&self, context: &AmlContext) -> Result<Arc<Spinlock<Vec<u8>>>, AmlError> {
         match self {
             AmlValue::Buffer(ref bytes) => Ok(bytes.clone()),
             // TODO: implement conversion of String and Integer to Buffer
             AmlValue::Field { .. } => self.read_field(context)?.as_buffer(context),
+            AmlValue::BufferField { .. } => self.read_buffer_field(context)?.as_buffer(context),
             _ => Err(AmlError::IncompatibleValueConversion { current: self.type_of(), target: AmlType::Buffer }),
         }
     }
@@ -394,6 +396,7 @@ impl AmlValue {
         // TODO: implement all of the rules
         match desired_type {
             AmlType::Integer => self.as_integer(context).map(|value| AmlValue::Integer(value)),
+            AmlType::Buffer => self.as_buffer(context).map(|value| AmlValue::Buffer(value)),
             AmlType::FieldUnit => panic!(
                 "Can't implicitly convert to FieldUnit. This must be special-cased by the caller for now :("
             ),
@@ -499,6 +502,83 @@ impl AmlValue {
         }
     }
 
+    pub fn read_buffer_field(&self, context: &AmlContext) -> Result<AmlValue, AmlError> {
+        use bitvec::view::BitView;
+
+        if let AmlValue::BufferField { buffer_data, offset, length } = self {
+            let offset = *offset as usize;
+            let length = *length as usize;
+            let inner_data = buffer_data.lock();
+
+            if (offset + length) > (inner_data.len() * 8) {
+                return Err(AmlError::BufferFieldIndexesOutOfBounds);
+            }
+
+            let bitslice = inner_data.view_bits::<bitvec::order::Lsb0>();
+            let bits = &bitslice[offset..(offset + length)];
+            if length > 64 {
+                Ok(AmlValue::Buffer(Arc::new(spinning_top::Spinlock::new(bits.as_raw_slice().to_vec()))))
+            } else {
+                let mut value = 0u64;
+                value.view_bits_mut::<bitvec::order::Lsb0>()[0..length].clone_from_bitslice(bits);
+                Ok(AmlValue::Integer(value))
+            }
+        } else {
+            Err(AmlError::IncompatibleValueConversion { current: self.type_of(), target: AmlType::BufferField })
+        }
+    }
+
+    pub fn write_buffer_field(&mut self, value: AmlValue, context: &mut AmlContext) -> Result<(), AmlError> {
+        use bitvec::view::BitView;
+
+        if let AmlValue::BufferField { buffer_data, offset, length } = self {
+            let offset = *offset as usize;
+            let length = *length as usize;
+            // TODO: check these against the size of the buffer to be written into
+            let mut inner_data = buffer_data.lock();
+            let bitslice = inner_data.view_bits_mut::<bitvec::order::Lsb0>();
+
+            match value {
+                AmlValue::Integer(value) => {
+                    /*
+                     * When an `Integer` is written into a `BufferField`, the entire contents are overwritten. If
+                     * it's smaller than the length of the buffer field, it's zero-extended. If it's larger, the
+                     * upper bits are truncated.
+                     */
+                    let bits_to_copy = cmp::min(length, 64);
+                    bitslice[offset..(offset + bits_to_copy)]
+                        .copy_from_bitslice(&value.to_le_bytes().view_bits()[..(bits_to_copy as usize)]);
+                    // Zero extend to the end of the buffer field
+                    bitslice[(offset + bits_to_copy)..(offset + length)].set_all(false);
+                    Ok(())
+                }
+                AmlValue::Boolean(value) => {
+                    bitslice.set(offset, value);
+                    Ok(())
+                }
+                AmlValue::Buffer(value) => {
+                    /*
+                     * When a `Buffer` is written into a `BufferField`, the entire contents are copied into the
+                     * field. If the buffer is smaller than the size of the buffer field, it is zero extended. If
+                     * the buffer is larger, the upper bits are truncated.
+                     * XXX: this behaviour is only explicitly defined in ACPI 2.0+. While undefined in ACPI 1.0,
+                     * we produce the same behaviour there.
+                     */
+                    let value_data = value.lock();
+                    let bits_to_copy = cmp::min(length, value_data.len() * 8);
+                    bitslice[offset..(offset + bits_to_copy)]
+                        .copy_from_bitslice(&value_data.view_bits()[..(bits_to_copy as usize)]);
+                    // Zero extend to the end of the buffer field
+                    bitslice[(offset + bits_to_copy)..(offset + length)].set_all(false);
+                    Ok(())
+                }
+                _ => Err(AmlError::TypeCannotBeWrittenToBufferField(value.type_of())),
+            }
+        } else {
+            Err(AmlError::IncompatibleValueConversion { current: self.type_of(), target: AmlType::BufferField })
+        }
+    }
+
     /// Logically compare two `AmlValue`s, according to the rules that govern opcodes like `DefLEqual`, `DefLLess`,
     /// etc. The type of `self` dictates the type that `other` will be converted to, and the method by which the
     /// values will be compared:
@@ -513,7 +593,7 @@ impl AmlValue {
 
         match self_inner.type_of() {
             AmlType::Integer => Ok(self.as_integer(context)?.cmp(&other.as_integer(context)?)),
-            AmlType::Buffer => Ok(self.as_buffer(context)?.cmp(&other.as_buffer(context)?)),
+            AmlType::Buffer => Ok(self.as_buffer(context)?.lock().cmp(&other.as_buffer(context)?.lock())),
             AmlType::String => Ok(self.as_string(context)?.cmp(&other.as_string(context)?)),
             typ => Err(AmlError::TypeCannotBeCompared(typ)),
         }
